package mobileAgent;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.UUID;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JTextArea;
import javax.swing.SwingUtilities;

import residenceAgent.ResidenceAgent;

public class DemoApp {
	private class LaunchRun implements Runnable {
		@Override
		public void run() {
			createAndShowGUI();
		};
	};

	private HashMap<String, GeneralMobileAgent> mobileAgentList = new HashMap<String, GeneralMobileAgent>();

	public void addMobileAgent(String key, GeneralMobileAgent mobileAgent) {
		mobileAgentList.put(key, mobileAgent);
	}

	private ResidenceAgent residenceAgent;

	public void setResidenceAgent(ResidenceAgent residenceAgent) {
		this.residenceAgent = residenceAgent;
	}

	private LaunchRun launchRun = new LaunchRun();

	private class SetTextRun implements Runnable {
		String text = "";

		public void setText(String text) {
			this.text = text;
		};

		public void run() {
			textArea.setText(text);
		};
	};

	private SetTextRun setTextRun = new SetTextRun();

	private class GetTextRun implements Runnable {
		String text = "";

		public String getText() {
			return text;
		};

		public void run() {
			text = textArea.getText();
		};
	};

	private GetTextRun getTextRun = new GetTextRun();
	private JTextArea textArea = new JTextArea();
	private JLabel label = new JLabel();
	final JRadioButton getNameListButton = new JRadioButton(
			"Get Pic-MetaData through All Nodes in Add-Hoc");
	final JRadioButton getIPAddressListButton = new JRadioButton(
			"Get List of IPAddresses");
	final JRadioButton oneStepTestButton = new JRadioButton(
			"Test oneStep transition mode");

	private ActionListener confirmButtonActionListener = new ConfirmButtonActionListener();

	class ConfirmButtonActionListener implements ActionListener {
		public void actionPerformed(ActionEvent arg0) {
			if (getNameListButton.isSelected()) {
				label.setText("mobile agent of Get Pic-MetaData through All Nodes in Add-Hoc selected");
				GeneralMobileAgent mobileAgentGetResidenceIDList = mobileAgentList
						.get("nameList");
				mobileAgentGetResidenceIDList
						.getApplicationSpecificMobileAgent("aaa", "3");
				MobileAgentInitializer.init(mobileAgentGetResidenceIDList,
						residenceAgent.getResidenceAgentName(), "allNodes",
						"userApp", null);
				mobileAgentGetResidenceIDList.setDelay(1000);
				mobileAgentGetResidenceIDList.setId(UUID.randomUUID()
						.toString());
				mobileAgentGetResidenceIDList.execute();
				mobileAgentGetResidenceIDList.transitToNextNode(true, true);
			} else if (getIPAddressListButton.isSelected()) {
				label.setText("mobile agent of Get IpAddress through All Nodes in Add-Hoc selected");
				GeneralMobileAgent mobileAgentGetResidenceIDList = mobileAgentList
						.get("ipAddressAgent");
				mobileAgentGetResidenceIDList
						.getApplicationSpecificMobileAgent("ipAddressService",
								"2");
				MobileAgentInitializer.init(mobileAgentGetResidenceIDList,
						residenceAgent.getResidenceAgentName(), "allNodes",
						"userApp", null);
				mobileAgentGetResidenceIDList.setDelay(50);
				mobileAgentGetResidenceIDList.setId(UUID.randomUUID()
						.toString());
				mobileAgentGetResidenceIDList.execute();
				mobileAgentGetResidenceIDList.transitToNextNode(true, true);
			} else if (oneStepTestButton.isSelected()) {
				label.setText("mobile agent tesing one-step move selected");
				GeneralMobileAgent mobileAgentPictureService = mobileAgentList
						.get("nameList");
				mobileAgentPictureService.getApplicationSpecificMobileAgent(
						"nameList", "3");
				HashMap<Object, Object> nodeMap = residenceAgent.getAgentList();
				Iterator<Object> iter = nodeMap.keySet().iterator();
				Collection<String> subSet = new ArrayList<String>();
				// add two nodes to the subset list
				int count = 0;
				while (iter.hasNext()) {
					String name = iter.next().toString();
					if (!name.equals(residenceAgent.getResidenceAgentName())) {
						subSet.add(name);
						count++;
						if (count == 1) {
							break;
						}
					}
				}
				MobileAgentInitializer.init(mobileAgentPictureService,
						residenceAgent.getResidenceAgentName(), "oneStep",
						"userApp", subSet);
				mobileAgentPictureService.setId(UUID.randomUUID().toString());
				mobileAgentPictureService.setDelay(1000);
				mobileAgentPictureService.execute();
				// if change the secode parameter to false, then no data is
				// taken back
				mobileAgentPictureService.transitToNextNode(true, true);
			} else {
				label.setText("please select one mobile agent to execute.");
			}
		}
	};

	public void setConfirmButtonListener(ActionListener listener) {
		confirmButtonActionListener = listener;
	}

	public ActionListener getConfirmButtonListener() {
		return confirmButtonActionListener;
	}

	private void createAndShowGUI() {
		// init page start panel
		JLabel headLabel = new JLabel("Please select a mobile agent: ");
		ButtonGroup group = new ButtonGroup();
		group.add(getNameListButton);
		group.add(getIPAddressListButton);
		group.add(oneStepTestButton);
		JButton confirmButton = new JButton("Confirm");
		confirmButton.addActionListener(confirmButtonActionListener);
		JPanel pageStartPanel = new JPanel();
		pageStartPanel
				.setLayout(new BoxLayout(pageStartPanel, BoxLayout.Y_AXIS));
		pageStartPanel.add(headLabel);
		pageStartPanel.add(getNameListButton);
		pageStartPanel.add(getIPAddressListButton);
		pageStartPanel.add(oneStepTestButton);
		pageStartPanel.add(confirmButton);
		// init center panel
		label.setText("mobile agent output information: ");
		textArea.setText("Please wait while mobile agent is \n"
				+ "executing through different residence agents...\n");
		textArea.setAutoscrolls(true);
		textArea.setRows(30);
		JPanel centerPanel = new JPanel();
		centerPanel.setLayout(new BoxLayout(centerPanel, BoxLayout.Y_AXIS));
		centerPanel.add(label);
		centerPanel.add(textArea);
		centerPanel.setBorder(BorderFactory.createTitledBorder(""));
		// init main UI
		JFrame frame = new JFrame("Get Residence Agent List");
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frame.getContentPane().setLayout(new BorderLayout());
		frame.getContentPane().add(BorderLayout.PAGE_START, pageStartPanel);
		frame.getContentPane().add(BorderLayout.CENTER, centerPanel);
		frame.setSize(400, 300);
		frame.setLocation(30, 40);
		frame.setVisible(true);
	};

	public void launch() {
		SwingUtilities.invokeLater(launchRun);
	};

	public void setText(String text) {
		setTextRun.setText(text);
		SwingUtilities.invokeLater(setTextRun);
	};

	public String getText() {
		SwingUtilities.invokeLater(getTextRun);
		try {
			Thread.sleep(1000);
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
		return getTextRun.getText();
	};

	public void mobileAgentComeBack(String id, String data) {
		String[] array = data.split("#takenAwayDataSplit#");
		String text = "mobile agent executing result: \n";
		for (int i = 0; i < array.length; i++) {
			String line = array[i];
			text = text + line + "\n";
		}
		setText(text);
		System.out.println("return result: " + text);
	};
}