package residenceAgent;

import java.io.ByteArrayOutputStream;
import java.io.PrintStream;
import java.util.HashMap;

import atVM.ApplicationExistsException;
import atVM.IApplication;
import edu.vub.at.IAT;

public class ResidenceAgent {
	private IAT intepreter;

	public ResidenceAgent(IAT iat) {
		this.intepreter = iat;
	}

	/**
	 * offers the id of the current residence agent.
	 * 
	 * @return the id of the current residence agent.
	 */
	public String getResidenceAgentName() {
		PrintStream sysout = System.out;
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		PrintStream ps = new PrintStream(baos, true);
		System.setOut(ps);
		intepreter.evalAndPrint("residenceAgent.getResidenceAgentName();",
				System.out);
		String residenceAgentName = baos.toString();
		System.setOut(sysout);
		residenceAgentName = residenceAgentName.replace("\"", "");
		if ("\n".equals(residenceAgentName.substring(residenceAgentName
				.length() - 1))) {
			residenceAgentName = residenceAgentName.replace("\n", "");
			residenceAgentName = residenceAgentName.trim();
		}
		return residenceAgentName;
	};

	public void register(String key, IApplication application)
			throws ApplicationExistsException {
		if (map.keySet().contains(key)) {
			ApplicationExistsException applicationExistsException = new ApplicationExistsException(
					key);
			applicationExistsException.setAppName(key);
			throw applicationExistsException;
		}
		intepreter.evalAndPrint("def application" + key + " := jlobby."
				+ application.getClass().getName() + ".new();");
		intepreter.evalAndPrint("residenceAgent.applicationRegistry.put(\""
				+ key + "\", application" + key + ");");
		map.put(key, application);
	}

	/**
	 * get local time of the current node;
	 * 
	 * @return the local time in the String format
	 */
	public String getCurrentTime() {
		PrintStream sysout = System.out;
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		PrintStream ps = new PrintStream(baos, true);
		System.setOut(ps);
		intepreter.evalAndPrint("residenceAgent.getCurrentTime();", System.out);
		String currentTime = baos.toString();
		System.setOut(sysout);
		currentTime = currentTime.replace("\r", "");
		currentTime = currentTime.replace("\n", "");
		currentTime = currentTime.replace("nil", "");
		currentTime = currentTime.replace("\"", "");
		return currentTime;
	};

	/**
	 * return all nodes in the network.
	 * 
	 * @return the hashmap contains the identifiers of all noes in the ad-hoc
	 *         network. The set of identifiers are in the keySet.
	 */
	public HashMap<Object, Object> getAgentList() {
		PrintStream sysout = System.out;
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		PrintStream ps = new PrintStream(baos, true);
		System.setOut(ps);
		intepreter.evalAndPrint(
				"def agentList := residenceAgent.getAgentList();"
						+ "def listIter := agentList.keySet().iterator();"
						+ "while: {listIter.hasNext()} do: {"
						+ "def id := listIter.next();" + "system.println(id);"
						+ "};", System.out);
		String listString = baos.toString();
		System.setOut(sysout);
		System.out.println("test result: " + listString);
		HashMap<Object, Object> agentList = new HashMap<Object, Object>();
		return agentList;
	}

	HashMap<String, IApplication> map = new HashMap<String, IApplication>();
}